@@ -7,7 +7,9 @@ Copyright (c) 2008 Jason Davies
 Licensed under the terms of the MIT License (see LICENSE.txt)
 """
 
+from datetime import datetime
 import os
+import re
 
 from django.conf import settings
 from django.core.cache import cache
@@ -18,15 +20,18 @@ from django.template.loader import render_to_string
 from django.utils.text import compress_string
 from django.utils.cache import patch_vary_headers, patch_response_headers
 
-from datetime import datetime
+import tinymce.settings
 
-TINYMCE_JS_ROOT = getattr(settings, 'TINYMCE_JS_ROOT', os.path.join(settings.MEDIA_ROOT, 'js/tiny_mce'))
-TINYMCE_BASE_URL = getattr(settings, 'TINYMCE_JS_URL', '%sjs/tiny_mce/tiny_mce.js' % settings.MEDIA_URL)
-TINYMCE_BASE_URL = TINYMCE_BASE_URL[:TINYMCE_BASE_URL.rfind('/')]
+safe_filename_re = re.compile("^[a-zA-Z][a-zA-Z0-9_/-]*$")
+
+def get_file_contents(filename):
+    base_path = tinymce.settings.JS_ROOT
+    if settings.DEBUG and settings.STATIC_ROOT:
+        from django.contrib.staticfiles import finders
+        base_path = finders.find('tiny_mce')
 
-def getFileContents(filename):
     try:
-        f = open(os.path.join(TINYMCE_JS_ROOT, filename))
+        f = open(os.path.join(base_path, filename))
         try:
             return f.read()
         finally:
@@ -53,7 +58,7 @@ def gzip_compressor(request):
 
     if not isJS:
         response.write(render_to_string('tinymce/tiny_mce_gzip.js', {
-            'TINYMCE_BASE_URL': TINYMCE_BASE_URL,
+            'base_url': tinymce.settings.JS_BASE_URL,
         }, context_instance=RequestContext(request)))
         return response
 
@@ -66,7 +71,7 @@ def gzip_compressor(request):
     cacheData = cache.get(cacheKey)
 
     if not cacheData is None:
-        if cacheData.has_key('ETag'):
+        if 'ETag' in cacheData:
             if_none_match = request.META.get('HTTP_IF_NONE_MATCH', None)
             if if_none_match == cacheData['ETag']:
                 response.status_code = 304
@@ -74,7 +79,7 @@ def gzip_compressor(request):
                 response['Content-Length'] = '0'
                 return response
 
-        if cacheData.has_key('Last-Modified'):
+        if 'Last-Modified' in cacheData:
             if_modified_since = request.META.get('HTTP_IF_MODIFIED_SINCE', None)
             if if_modified_since == cacheData['Last-Modified']:
                 response.status_code = 304
@@ -82,32 +87,37 @@ def gzip_compressor(request):
                 response['Content-Length'] = '0'
                 return response
 
-    # Add core, with baseURL added
-    content.append(getFileContents("tiny_mce%s.js" % suffix).replace("tinymce._init();", "tinymce.baseURL='%s';tinymce._init();" % TINYMCE_BASE_URL))
+    content.append("var tinyMCEPreInit={base:'%s',suffix:''};" % tinymce.settings.JS_BASE_URL);
 
-    # Patch loading functions
-    content.append("tinyMCE_GZ.start();")
+    # Add core
+    files = ["tiny_mce"]
 
     # Add core languages
     for lang in languages:
-        content.append(getFileContents("langs/%s.js" % lang))
-
-    # Add themes
-    for theme in themes:
-        content.append(getFileContents("themes/%s/editor_template%s.js"%  (theme, suffix)))
-
-        for lang in languages:
-            content.append(getFileContents("themes/%s/langs/%s.js" % (theme, lang)))
+        files.append("langs/%s" % lang)
 
     # Add plugins
     for plugin in plugins:
-        content.append(getFileContents("plugins/%s/editor_plugin%s.js" % (plugin, suffix)))
+        files.append("plugins/%s/editor_plugin%s" % (plugin, suffix))
 
         for lang in languages:
-            content.append(getFileContents("plugins/%s/langs/%s.js" % (plugin, lang)))
+            files.append("plugins/%s/langs/%s" % (plugin, lang))
+
+    # Add themes
+    for theme in themes:
+        files.append("themes/%s/editor_template%s" % (theme, suffix))
+
+        for lang in languages:
+            files.append("themes/%s/langs/%s" % (theme, lang))
+
+    for f in files:
+        # Check for unsafe characters
+        if not safe_filename_re.match(f):
+            continue
+        content.append(get_file_contents("%s.js" % f))
 
     # Restore loading functions
-    content.append("tinyMCE_GZ.end();")
+    content.append('tinymce.each("%s".split(","), function(f){tinymce.ScriptLoader.markDone(tinyMCE.baseURL+"/"+f+".js");});' % ",".join(files));
 
     # Compress
     if compress:
@@ -120,6 +130,6 @@ def gzip_compressor(request):
     patch_response_headers(response, timeout)
     cache.set(cacheKey, {
         'Last-Modified': response['Last-Modified'],
-        'ETag': response['ETag'],
+        'ETag': response.get('ETag', ''),
     })
     return response
