@@ -1,7 +1,10 @@
 # Copyright (c) 2008 Joost Cassee
 # Licensed under the terms of the MIT License (see LICENSE.txt)
 
-from django.conf.urls.defaults import *
+try:
+    from django.conf.urls import url, patterns
+except:
+    from django.conf.urls.defaults import url, patterns
 
 urlpatterns = patterns('tinymce.views',
     url(r'^js/textareas/(?P<name>.+)/$', 'textareas_js', name='tinymce-js'),
@@ -10,4 +13,5 @@ urlpatterns = patterns('tinymce.views',
     url(r'^flatpages_link_list/$', 'flatpages_link_list'),
     url(r'^compressor/$', 'compressor', name='tinymce-compressor'),
     url(r'^filebrowser/$', 'filebrowser', name='tinymce-filebrowser'),
+    url(r'^preview/(?P<name>.+)/$', 'preview', name='tinymce-preview'),
 )
