@@ -47,52 +47,50 @@ class TinyMCE(forms.Textarea):
     the current Django language, the others from the 'content_language'
     parameter.
     """
-
     def __init__(self, content_language=None, attrs=None, mce_attrs=None):
         super(TinyMCE, self).__init__(attrs)
-        if mce_attrs is None:
-            mce_attrs = {}
+        mce_attrs = mce_attrs or {}
         self.mce_attrs = mce_attrs
+        if not 'mode' in self.mce_attrs:
+            self.mce_attrs['mode'] = 'exact'
+        self.mce_attrs['strict_loading_mode'] = 1
         if content_language is None:
             content_language = mce_attrs.get('language', None)
         self.content_language = content_language
 
-    def render(self, name, value, attrs=None):
-        if value is None: value = ''
-        value = smart_unicode(value)
-        final_attrs = self.build_attrs(attrs)
-        final_attrs['name'] = name
-        assert 'id' in final_attrs, "TinyMCE widget attributes must contain 'id'"
-
+    def get_mce_config(self, attrs):
         mce_config = tinymce.settings.DEFAULT_CONFIG.copy()
         mce_config.update(get_language_config(self.content_language))
         if tinymce.settings.USE_FILEBROWSER:
             mce_config['file_browser_callback'] = "djangoFileBrowser"
         mce_config.update(self.mce_attrs)
-        if not 'mode' in mce_config:
-            mce_config['mode'] = 'exact'
         if mce_config['mode'] == 'exact':
-            mce_config['elements'] = final_attrs['id']
-        mce_config['strict_loading_mode'] = 1
-        
+            mce_config['elements'] = attrs['id']
+        return mce_config
+
+    def get_mce_json(self, mce_config):
         # Fix for js functions
         js_functions = {}
-        for k in ('paste_preprocess','paste_postprocess'):
+        for k in ('paste_preprocess', 'paste_postprocess'):
             if k in mce_config:
                js_functions[k] = mce_config[k]
                del mce_config[k]
         mce_json = json.dumps(mce_config)
-
-        pos = final_attrs['id'].find('__prefix__')
-        if pos != -1:
-            mce_json = mce_json.replace('"%s"' % final_attrs['id'], 'elements')
-
         for k in js_functions:
             index = mce_json.rfind('}')
             mce_json = mce_json[:index]+', '+k+':'+js_functions[k].strip()+mce_json[index:]
-            
+        return mce_json
 
-        html = ['<textarea%s>%s</textarea>' % (flatatt(final_attrs), escape(value))]
+    def render(self, name, value, attrs=None):
+        if value is None:
+            value = ''
+        value = smart_unicode(value)
+        final_attrs = self.build_attrs(attrs)
+        final_attrs['name'] = name
+        final_attrs['class'] = 'tinymce'
+        assert 'id' in final_attrs, "TinyMCE widget attributes must contain 'id'"
+        mce_config = self.get_mce_config(final_attrs)
+        mce_json = self.get_mce_json(mce_config)
         if tinymce.settings.USE_COMPRESSOR:
             compressor_config = {
                 'plugins': mce_config.get('plugins', ''),
@@ -101,31 +99,9 @@ class TinyMCE(forms.Textarea):
                 'diskcache': True,
                 'debug': False,
             }
-            compressor_json = json.dumps(compressor_config)
-            html.append('<script type="text/javascript">tinyMCE_GZ.init(%s)</script>' % compressor_json)
-            
-        if pos != -1:
-            html.append('''<script type="text/javascript">
-setTimeout(function () {
-    var id = '%s';
-    
-    if (typeof(window._tinymce_inited) == 'undefined') {
-        window._tinymce_inited = [];
-    }
-    
-    if (typeof(window._tinymce_inited[id]) == 'undefined') {
-        window._tinymce_inited[id] = true;
-    } else {
-        var elements = id.replace(/__prefix__/, parseInt(document.getElementById('%sTOTAL_FORMS').value) - 1);
-        if (document.getElementById(elements)) {
-            tinymce.init(%s);
-        }
-    }
-}, 0);
-</script>''' % (final_attrs['id'], final_attrs['id'][0:pos], mce_json))
-        else:
-            html.append('<script type="text/javascript">tinyMCE.init(%s)</script>' % mce_json)
-
+            final_attrs['data-mce-gz-conf'] = json.dumps(compressor_config)
+        final_attrs['data-mce-conf'] = mce_json
+        html = ['<textarea%s>%s</textarea>' % (flatatt(final_attrs), escape(value))]
         return mark_safe('\n'.join(html))
 
     def _media(self):
@@ -135,11 +111,12 @@ setTimeout(function () {
             js = [tinymce.settings.JS_URL]
         if tinymce.settings.USE_FILEBROWSER:
             js.append(reverse('tinymce-filebrowser'))
+        js.append('django_tinymce/init_tinymce.js')
         return forms.Media(js=js)
     media = property(_media)
 
 
-class AdminTinyMCE(admin_widgets.AdminTextareaWidget, TinyMCE):
+class AdminTinyMCE(TinyMCE, admin_widgets.AdminTextareaWidget):
     pass
 
 
